/*
 * @Copyright (c) 2018 缪聪(mcg-helper@qq.com)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");  
 * you may not use this file except in compliance with the License.  
 * You may obtain a copy of the License at  
 *     
 *     http://www.apache.org/licenses/LICENSE-2.0  
 *     
 * Unless required by applicable law or agreed to in writing, software  
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  
 * See the License for the specific language governing permissions and  
 * limitations under the License.
 */

package com.mcg.plugin.endesign;

import java.security.InvalidKeyException;
import java.security.KeyFactory;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;
import java.security.interfaces.RSAPrivateKey;
import java.security.interfaces.RSAPublicKey;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.PKCS8EncodedKeySpec;
import java.security.spec.X509EncodedKeySpec;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class RSAEncrypt {
	
	private static Logger logger = LoggerFactory.getLogger(RSAEncrypt.class);
    /**
     * 字节数据转字符串专用集合
     */
    private static final char[] HEX_CHAR = { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f' };

    /**
     * 随机生成密钥对
     */
    public static String genKeyPair() {
        // KeyPairGenerator类用于生成公钥和私钥对，基于RSA算法生成对象
        KeyPairGenerator keyPairGen = null;
        try {
            keyPairGen = KeyPairGenerator.getInstance("RSA");
        } catch (NoSuchAlgorithmException e) {
        	logger.error(e.getMessage());
        }
        // 初始化密钥对生成器，密钥大小为96-2048位
        keyPairGen.initialize(1024, new SecureRandom());
        // 生成一个密钥对，保存在keyPair中
        KeyPair keyPair = keyPairGen.generateKeyPair();
        // 得到私钥
        RSAPrivateKey privateKey = (RSAPrivateKey) keyPair.getPrivate();
        // 得到公钥
        RSAPublicKey publicKey = (RSAPublicKey) keyPair.getPublic();
        // 得到公钥字符串
        String publicKeyString = Base64.encode(publicKey.getEncoded());
        // 得到私钥字符串
        String privateKeyString = Base64.encode(privateKey.getEncoded());
        // 将密钥对写入到文件
        return "{publicKey:" + publicKeyString + ", privateKey:" + privateKeyString + "}";
    }

    /**
     * 从字符串中加载公钥
     * 
     * @param publicKeyStr 公钥数据字符串
     * @throws Exception 加载公钥时产生的异常
     */
    public static RSAPublicKey loadPublicKeyByStr(String publicKeyStr) throws Exception {
        try {
            byte[] buffer = Base64.decode(publicKeyStr);
            KeyFactory keyFactory = KeyFactory.getInstance("RSA");
            X509EncodedKeySpec keySpec = new X509EncodedKeySpec(buffer);
            return (RSAPublicKey) keyFactory.generatePublic(keySpec);
        } catch (NoSuchAlgorithmException e) {
        	logger.error("无此算法，异常信息：{}", e.getMessage());
            throw new Exception("无此算法");
        } catch (InvalidKeySpecException e) {
        	logger.error("公钥非法，异常信息：{}", e.getMessage());
            throw new Exception("公钥非法");
        } catch (NullPointerException e) {
        	logger.error("公钥数据为空，异常信息：{}", e.getMessage());
            throw new Exception("公钥数据为空");
        }
    }

    public static RSAPrivateKey loadPrivateKeyByStr(String privateKeyStr) throws Exception {
        try {
            byte[] buffer = Base64.decode(privateKeyStr);
            PKCS8EncodedKeySpec keySpec = new PKCS8EncodedKeySpec(buffer);
            KeyFactory keyFactory = KeyFactory.getInstance("RSA");
            return (RSAPrivateKey) keyFactory.generatePrivate(keySpec);
        } catch (NoSuchAlgorithmException e) {
        	logger.error("无此算法，异常信息：{}", e.getMessage());
            throw new Exception("无此算法");
        } catch (InvalidKeySpecException e) {
        	logger.error("私钥非法，异常信息：{}", e.getMessage());
            throw new Exception("私钥非法");
        } catch (NullPointerException e) {
        	logger.error("私钥数据为空，异常信息：{}", e.getMessage());
            throw new Exception("私钥数据为空");
        }
    }

    /**
     * 公钥加密过程
     * 
     * @param publicKey 公钥
     * @param plainTextData 明文数据
     * @return
     * @throws Exception 加密过程中的异常信息
     */
    public static byte[] encrypt(RSAPublicKey publicKey, byte[] plainTextData) throws Exception {
        if (publicKey == null) {
            throw new Exception("加密公钥为空, 请设置");
        }
        Cipher cipher = null;
        try {
            // 使用默认RSA
            cipher = Cipher.getInstance("RSA");
            // cipher= Cipher.getInstance("RSA", new BouncyCastleProvider());
            cipher.init(Cipher.ENCRYPT_MODE, publicKey);
            byte[] output = cipher.doFinal(plainTextData);
            return output;
        } catch (NoSuchAlgorithmException e) {
        	logger.error("无此加密算法，异常信息：{}", e.getMessage());
            throw new Exception("无此加密算法");
        } catch (NoSuchPaddingException e) {
        	logger.error(e.getMessage());
            return null;
        } catch (InvalidKeyException e) {
        	logger.error("加密公钥非法，异常信息：{}", e.getMessage());
            throw new Exception("加密公钥非法,请检查");
        } catch (IllegalBlockSizeException e) {
        	logger.error("明文长度非法，异常信息：{}", e.getMessage());
            throw new Exception("明文长度非法");
        } catch (BadPaddingException e) {
        	logger.error("明文数据已损坏，异常信息：{}", e.getMessage());
            throw new Exception("明文数据已损坏");
        }
    }

    /**
     * 私钥加密过程
     * 
     * @param privateKey 私钥
     * @param plainTextData 明文数据
     * @return
     * @throws Exception 加密过程中的异常信息
     */
    public static byte[] encrypt(RSAPrivateKey privateKey, byte[] plainTextData) throws Exception {
        if (privateKey == null) {
            throw new Exception("加密私钥为空, 请设置");
        }
        Cipher cipher = null;
        try {
            // 使用默认RSA
            cipher = Cipher.getInstance("RSA");
            cipher.init(Cipher.ENCRYPT_MODE, privateKey);
            byte[] output = cipher.doFinal(plainTextData);
            return output;
        } catch (NoSuchAlgorithmException e) {
        	logger.error("无此加密算法，异常信息：{}", e.getMessage());
            throw new Exception("无此加密算法");
        } catch (NoSuchPaddingException e) {
        	logger.error(e.getMessage());
            return null;
        } catch (InvalidKeyException e) {
        	logger.error("加密私钥非法，异常信息：{}", e.getMessage());
            throw new Exception("加密私钥非法,请检查");
        } catch (IllegalBlockSizeException e) {
        	logger.error("明文长度非法，异常信息：{}", e.getMessage());
            throw new Exception("明文长度非法");
        } catch (BadPaddingException e) {
        	logger.error("明文数据已损坏，异常信息：{}", e.getMessage());
            throw new Exception("明文数据已损坏");
        }
    }

    /**
     * 私钥解密过程
     * 
     * @param privateKey 私钥
     * @param cipherData 密文数据
     * @return 明文
     * @throws Exception 解密过程中的异常信息
     */
    public static byte[] decrypt(RSAPrivateKey privateKey, byte[] cipherData) throws Exception {
        if (privateKey == null) {
            throw new Exception("解密私钥为空, 请设置");
        }
        Cipher cipher = null;
        try {
            // 使用默认RSA
            cipher = Cipher.getInstance("RSA");
            // cipher= Cipher.getInstance("RSA", new BouncyCastleProvider());
            cipher.init(Cipher.DECRYPT_MODE, privateKey);
            byte[] output = cipher.doFinal(cipherData);
            return output;
        } catch (NoSuchAlgorithmException e) {
        	logger.error("无此解密算法，异常信息：{}", e.getMessage());
            throw new Exception("无此解密算法");
        } catch (NoSuchPaddingException e) {
        	logger.error(e.getMessage());
            return null;
        } catch (InvalidKeyException e) {
        	logger.error("解密私钥非法，异常信息：{}", e.getMessage());
            throw new Exception("解密私钥非法,请检查");
        } catch (IllegalBlockSizeException e) {
        	logger.error("密文长度非法，异常信息：{}", e.getMessage());
            throw new Exception("密文长度非法");
        } catch (BadPaddingException e) {
        	logger.error("密文数据已损坏，异常信息：{}", e.getMessage());
            throw new Exception("密文数据已损坏");
        }
    }

    /**
     * 公钥解密过程
     * 
     * @param publicKey 公钥
     * @param cipherData 密文数据
     * @return 明文
     * @throws Exception 解密过程中的异常信息
     */
    public static byte[] decrypt(RSAPublicKey publicKey, byte[] cipherData) throws Exception {
        if (publicKey == null) {
            throw new Exception("解密公钥为空, 请设置");
        }
        Cipher cipher = null;
        try {
            // 使用默认RSA
            cipher = Cipher.getInstance("RSA");
            // cipher= Cipher.getInstance("RSA", new BouncyCastleProvider());
            cipher.init(Cipher.DECRYPT_MODE, publicKey);
            byte[] output = cipher.doFinal(cipherData);
            return output;
        } catch (NoSuchAlgorithmException e) {
        	logger.error("无此解密算法，异常信息：{}", e.getMessage());
            throw new Exception("无此解密算法");
        } catch (NoSuchPaddingException e) {
        	logger.error(e.getMessage());
            return null;
        } catch (InvalidKeyException e) {
        	logger.error("解密公钥非法，异常信息：{}", e.getMessage());
            throw new Exception("解密公钥非法,请检查");
        } catch (IllegalBlockSizeException e) {
        	logger.error("密文长度非法，异常信息：{}", e.getMessage());
            throw new Exception("密文长度非法");
        } catch (BadPaddingException e) {
        	logger.error("密文数据已损坏，异常信息：{}", e.getMessage());
            throw new Exception("密文数据已损坏");
        }
    }

    /**
     * 字节数据转十六进制字符串
     * 
     * @param data 输入数据
     * @return 十六进制内容
     */
    public static String byteArrayToString(byte[] data) {
        StringBuilder stringBuilder = new StringBuilder();
        for (int i = 0; i < data.length; i++) {
            // 取出字节的高四位 作为索引得到相应的十六进制标识符 注意无符号右移
            stringBuilder.append(HEX_CHAR[(data[i] & 0xf0) >>> 4]);
            // 取出字节的低四位 作为索引得到相应的十六进制标识符
            stringBuilder.append(HEX_CHAR[(data[i] & 0x0f)]);
            if (i < data.length - 1) {
                stringBuilder.append(' ');
            }
        }
        return stringBuilder.toString();
    }
}
